<?php
/**
 * Roundcube Plus xframework plugin
 *
 * Copyright 2016, Tecorama LLC.
 *
 * @license Commercial. See the file LICENSE for details.
 */

require_once(__DIR__ . "/../common/Test.php");
require_once(__DIR__ . "/../common/Utils.php");

use XFramework\Utils as Utils;

class UtilsTest extends XFramework\Test
{
    public function __construct() {
        parent::__construct();

        // get temp dir
        $this->tempDir = $this->rcmail->config->get("temp_dir");
        $this->assertTrue($this->tempDir != false);
        $this->tempDir = Utils::addSlash($this->tempDir);
    }

    public function testGetRemoteAddr()
    {
        $ip = "5.157.7.42";
        $_SERVER['REMOTE_ADDR'] = $ip;
        $this->assertEquals(Utils::getRemoteAddr(), $ip);
        unset($_SERVER['REMOTE_ADDR']);

        $this->assertEquals(Utils::getRemoteAddr(), "");

        $this->rcmail->config->set("remote_addr_key", "ALTERNATE_REMOTE_ADDR");
        $_SERVER['ALTERNATE_REMOTE_ADDR'] = "82.102.20.178";
        $this->assertEquals(Utils::getRemoteAddr(), "82.102.20.178");
        unset($_SERVER['ALTERNATE_REMOTE_ADDR']);
    }

    public function testSizeToString()
    {
        $this->assertEquals(Utils::sizeToString("hello"), "-");
        $this->assertEquals(Utils::sizeToString(12345678), "12 MB");
    }

    public function testShortenString()
    {
        $this->assertEquals(Utils::shortenString("this is a long string", 10), "this is a...");
        $this->assertEquals(Utils::shortenString("hello", 10), "hello");
    }

    public function testStructuredDirectory()
    {
        $this->assertEquals(Utils::structuredDirectory(1234, -5, 10), "000/012/");
    }

    public function testEnsureFileName()
    {
        $this->assertEquals(Utils::ensureFileName("hello/\:?*+%|\"<>there"), "hello_there");
    }

    public function testUniqueFileName()
    {
        $this->assertMatchesRegularExpression('/prefix_.*\.png/', Utils::uniqueFileName("/home/maya", "png", "prefix_"));
    }

    public function testExt()
    {
        $this->assertEquals(Utils::ext("hello.png"), "png");
    }

    public function testMakeDir()
    {
        $dir = $this->tempDir . "test_dir";
        Utils::removeDir($dir);

        // test creation
        $this->assertTrue(Utils::makeDir($dir));
        // test if true if already exists
        $this->assertTrue(Utils::makeDir($dir));
    }

    public function testRemoveDir()
    {
        $dir = $this->tempDir . "test_dir";
        $file = "$dir/directory/file";
        $symTarget = $this->tempDir . "test_file";
        $symLink = "$dir/directory/sym";


        $this->assertTrue(Utils::removeDir("$dir/some-non-existent-directory"));

        $this->createTestFiles($dir, $file, $symTarget, $symLink);
        $this->assertTrue(Utils::removeDir($dir, false, true));
        $this->assertFalse(file_exists($file));
        $this->assertFalse(file_exists($symLink));
        $this->assertTrue(file_exists($symTarget));

        $this->createTestFiles($dir, $file, $symTarget, $symLink);
        $this->assertTrue(Utils::removeDir($dir, true, true));
        $this->assertFalse(file_exists($file));
        $this->assertFalse(file_exists($symLink));
        $this->assertFalse(file_exists($symTarget));

        $this->createTestFiles($dir, $file, $symTarget, $symLink);
        $this->assertTrue(Utils::removeDir($dir, true, false));
        $this->assertFalse(file_exists($dir));
    }

    public function testRemoveSlash()
    {
        $this->assertEquals(Utils::removeSlash("/test/file/"), "/test/file");
        $this->assertEquals(Utils::removeSlash("/test/file\\"), "/test/file");
    }

    public function testAddSlash()
    {
        $this->assertEquals(Utils::addSlash("/test/file"), "/test/file/");
    }

    public function testRandomToken()
    {
        $this->assertMatchesRegularExpression('/[0-9,a-z]{32}/', Utils::randomToken());
        $this->assertMatchesRegularExpression('/[0-9,a-z]{8}/', Utils::randomToken(8));
    }

    public function testEncodeId()
    {
        xrc()->config->set("des_key", "2587750ebe02dfc4438cda6a");
        $this->assertEquals(Utils::encodeId(1), "935fcab7");
        $this->assertEquals(Utils::encodeId(999999999), "cefa94b5");
    }

    public function testDecodeId()
    {
        xrc()->config->set("des_key", "2587750ebe02dfc4438cda6a");
        $this->assertEquals(Utils::decodeId("935fcab7"), 1);
        $this->assertEquals(Utils::decodeId("cefa94b5"), 999999999);
    }

    public function testEncodeUrlAction()
    {
        $action = "some_action";
        $data = "some_data";
        $output = Utils::encodeUrlAction($action, $data);
        $decoded_action = Utils::decodeUrlAction($output, $decoded_data);
        $this->assertEquals($action, $decoded_action);
        $this->assertEquals($data, $decoded_data);
    }

    public function testDecodeUrlAction()
    {
        $action = Utils::decodeUrlAction("JrTQemsL1kiOYbvmgjh-xIN-990CEQ_RjgdOT7DaTwT8hZjLoHVMkEpD3pPdB1-PYrZ5sg8b4Yg", $data);
        $this->assertEquals($action, "some_action");
        $this->assertEquals($data, "some_data");
        $this->assertFalse(Utils::decodeUrlAction("abc", $data));
    }

    public function testPack()
    {
        $this->assertTrue(strpos(Utils::pack(["abc" => 44]), "6472056011600") !== false);
    }

    public function testEncodeBinary()
    {
        $this->assertEquals(Utils::encodeBinary("some_data"), "c29tZV9kYXRh");
    }

    public function testDecodeBinary()
    {
        $this->assertEquals(Utils::decodeBinary("c29tZV9kYXRh"), "some_data");
    }

    public function testLoadConfigFile()
    {
        $configFile = $this->tempDir . "test_config.php";
        file_put_contents($configFile, "<?php \$config = ['abc' => 44];");
        $config = Utils::loadConfigFile($configFile);
        unlink($configFile);
        $this->assertEquals($config['abc'], 44);
    }

    public function testXlog()
    {
        $file = rtrim(RCUBE_INSTALL_PATH, "/") . "/logs/unit_test.log";
        file_exists($file) && unlink($file);
        Utils::xlog("some_error", "unit_test.log");
        $text = file_get_contents($file);
        unlink($file);
        $this->assertEquals($text, "[" . date("Y-m-d H:i:s") . "] some_error\n");
    }

    public function testRemoveVarsFromUrl()
    {
        $url = "https://roundcubeplus.com";
        $this->assertEquals(Utils::removeVarsFromUrl("hello", "$url?hello=1&abc=2"), "$url?abc=2");
        $this->assertEquals(Utils::removeVarsFromUrl("hello", $url), $url);
    }

    public function testAddVarsToUrl()
    {
        $url = "https://roundcubeplus.com";
        $this->assertEquals(Utils::addVarsToUrl(["hello" => 44], $url), "$url?hello=44");
        $this->assertEquals(Utils::addVarsToUrl(["hello" => 44], "$url?abc=2"), "$url?abc=2&hello=44");
        $this->assertEquals(Utils::addVarsToUrl([], $url), $url);
    }

    public function testUuid()
    {
        $this->assertEquals(preg_match('/^[a-f\d]{8}-[a-f\d]{4}-4[a-f\d]{3}-[89ab][a-f\d]{3}-[a-f\d]{12}$/i', Utils::uuid()), 1);
    }

    public function testSanitizeHtml()
    {
        $this->assertEquals(
            Utils::sanitizeHtml("<p><script></script><em onclick='alert(1)'>a</em><a href='#' onclick='alert(1)'>b</a></p>"),
            '<p><em>a</em><a href="#" target="_blank">b</a></p>'
        );
    }

    public function testIpMatch()
    {
        $this->assertFalse(Utils::ipMatch("xxx", "45.133.193.83"));
        $this->assertFalse(Utils::ipMatch("45.133.193.83", "xxx"));

        $this->assertTrue(Utils::ipMatch("45.133.193.83", "45.133.193.83"));
        $this->assertFalse(Utils::ipMatch("45.133.193.82", "45.133.193.83"));
        $this->assertTrue(Utils::ipMatch("7a4b:cfe:3afb:680c:4245:80d4:c420:fd76", "7a4b:cfe:3afb:680c:4245:80d4:c420:fd76"));
        $this->assertFalse(Utils::ipMatch("7a4b:cfe:3afb:680c:4245:80d4:c420:fd75", "7a4b:cfe:3afb:680c:4245:80d4:c420:fd76"));

        // 45.133.193.0 - 45.133.193.255
        $this->assertTrue(Utils::ipMatch("45.133.193.0", "45.133.193.83/24"));
        $this->assertTrue(Utils::ipMatch("45.133.193.83", "45.133.193.83/24"));
        $this->assertTrue(Utils::ipMatch("45.133.193.255", "45.133.193.83/24"));
        $this->assertFalse(Utils::ipMatch("45.133.192.255", "45.133.193.83/24"));
        $this->assertFalse(Utils::ipMatch("45.133.194.0", "45.133.193.83/24"));
        $this->assertTrue(Utils::ipMatch("45.133.193.0", "45.133.193.0 - 45.133.193.255"));
        $this->assertTrue(Utils::ipMatch("45.133.193.83", "45.133.193.0 - 45.133.193.255"));
        $this->assertTrue(Utils::ipMatch("45.133.193.255", "45.133.193.0 - 45.133.193.255"));
        $this->assertFalse(Utils::ipMatch("45.133.192.255", "45.133.193.0 - 45.133.193.255"));
        $this->assertFalse(Utils::ipMatch("45.133.194.0", "45.133.193.0 - 45.133.193.255"));

        // 45.128.0.0 - 45.135.255.255
        $this->assertTrue(Utils::ipMatch("45.128.0.0", "45.133.193.83/13"));
        $this->assertTrue(Utils::ipMatch("45.135.100.100", "45.133.193.83/13"));
        $this->assertTrue(Utils::ipMatch("45.135.255.255", "45.133.193.83/13"));
        $this->assertFalse(Utils::ipMatch("45.127.0.0", "45.133.193.83/13"));
        $this->assertFalse(Utils::ipMatch("45.136.0.0", "45.133.193.83/13"));
        $this->assertTrue(Utils::ipMatch("45.128.0.0", "45.128.0.0 - 45.135.255.255"));
        $this->assertTrue(Utils::ipMatch("45.135.100.100", "45.128.0.0 - 45.135.255.255"));
        $this->assertTrue(Utils::ipMatch("45.135.255.255", "45.128.0.0 - 45.135.255.255"));
        $this->assertFalse(Utils::ipMatch("45.127.0.0", "45.128.0.0 - 45.135.255.255"));
        $this->assertFalse(Utils::ipMatch("45.136.0.0", "45.128.0.0 - 45.135.255.255"));

        // 2001:4860:0000:0000:0000:0000:0000:0000 - 2001:4860:ffff:ffff:ffff:ffff:ffff:ffff
        $this->assertTrue(Utils::ipMatch("2001:4860:0000:0000:0000:0000:0000:8888", "2001:4860:0000::8888/32"));
        $this->assertTrue(Utils::ipMatch("2001:4860:0000:0000:0000:0000:0000:8888", "2001:4860:0000:0000:0000:0000:0000:8888 - 2001:4860:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF"));
        $this->assertFalse(Utils::ipMatch("2001:4850:0000:0000:0000:0000:0000:8888", "2001:4860:0000::8888/32"));
    }

    private function createTestFiles($dir, $file, $symTarget, $symLink)
    {
        Utils::makeDir("$dir/directory");
        file_put_contents($file, "hello");
        file_put_contents($symTarget, "hello");
        symlink($symTarget, $symLink);
        $this->assertTrue(file_exists($file));
        $this->assertTrue(file_exists($symTarget));
        $this->assertTrue(file_exists($symLink));
    }

}